@@ -1,4 +1,4 @@
-// Copyright (C) 2010-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2010-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,6 +7,8 @@
 #ifndef ISC_DATA_H
 #define ISC_DATA_H 1
 
+#include <util/bigints.h>
+
 #include <iostream>
 #include <map>
 #include <stdexcept>
@@ -126,11 +128,31 @@ public:
         return (position);
     }
 
+    /// @brief The types that an Element can hold
+    ///
+    /// Some of these types need to match their associated integer from the
+    /// parameter_data_type database table, so let the enums be explicitly
+    /// mapped to integers, to reduce the chance of messing up.
+    ///
+    /// any is a special type used in list specifications, specifying that the
+    /// elements can be of any type.
+    enum types {
+        integer = 0,
+        real = 1,
+        boolean = 2,
+        null = 3,
+        string = 4,
+        bigint = 5,
+        list = 6,
+        map = 7,
+        any = 8,
+    };
+
 private:
     // technically the type could be omitted; is it useful?
     // should we remove it or replace it with a pure virtual
     // function getType?
-    int type_;
+    types type_;
 
     /// @brief Position of the element in the configuration string.
     Position position_;
@@ -143,21 +165,17 @@ protected:
     /// @param pos Structure holding position of the value of the data element.
     /// It comprises the line number and the position within this line. The values
     /// held in this structure are used for error logging purposes.
-    Element(int t, const Position& pos = ZERO_POSITION())
+    Element(types t, const Position& pos = ZERO_POSITION())
         : type_(t), position_(pos) {
     }
 
 
 public:
-
-    // any is a special type used in list specifications, specifying
-    // that the elements can be of any type
-    enum types { integer, real, boolean, null, string, list, map, any };
     // base class; make dtor virtual
     virtual ~Element() {};
 
     /// @return the type of this element
-    int getType() const { return (type_); }
+    types getType() const { return (type_); }
 
     /// @brief Returns position where the data element's value starts in a
     /// configuration string.
@@ -197,7 +215,8 @@ public:
 
     /// @name pure virtuals, every derived class must implement these
 
-    /// @return true if the other ElementPtr has the same type and value
+    /// @return true if the other ElementPtr has the same value and the same
+    /// type (or a different and compatible type), false otherwise.
     virtual bool equals(const Element& other) const = 0;
 
     /// Converts the Element to JSON format and appends it to
@@ -214,6 +233,9 @@ public:
     //@{
     virtual int64_t intValue() const
     { throwTypeError("intValue() called on non-integer Element"); };
+    virtual isc::util::int128_t bigIntValue() const {
+        throwTypeError("bigIntValue() called on non-big-integer Element");
+    }
     virtual double doubleValue() const
     { throwTypeError("doubleValue() called on non-double Element"); };
     virtual bool boolValue() const
@@ -258,6 +280,7 @@ public:
     ///        long long int, long int and int.
     //@{
     virtual bool setValue(const long long int v);
+    virtual bool setValue(const isc::util::int128_t& v);
     bool setValue(const long int i) { return (setValue(static_cast<long long int>(i))); };
     bool setValue(const int i) { return (setValue(static_cast<long long int>(i))); };
     virtual bool setValue(const double v);
@@ -376,6 +399,8 @@ public:
     static ElementPtr create(const Position& pos = ZERO_POSITION());
     static ElementPtr create(const long long int i,
                              const Position& pos = ZERO_POSITION());
+    static ElementPtr create(const isc::util::int128_t& i,
+                             const Position& pos = ZERO_POSITION());
     static ElementPtr create(const int i,
                              const Position& pos = ZERO_POSITION());
     static ElementPtr create(const long int i,
@@ -601,6 +626,48 @@ public:
     bool equals(const Element& other) const;
 };
 
+/// @brief Wrapper over int128_t
+class BigIntElement : public Element {
+    using int128_t = isc::util::int128_t;
+    using Element::getValue;
+    using Element::setValue;
+
+public:
+    /// @brief Constructor
+    BigIntElement(const int128_t& v, const Position& pos = ZERO_POSITION())
+        : Element(bigint, pos), i_(v) {
+    }
+
+    /// @brief Retrieve the underlying big integer value.
+    ///
+    /// @return the underlying value
+    int128_t bigIntValue() const override {
+        return (i_);
+    }
+
+    /// @brief Sets the underlying big integer value.
+    ///
+    /// @return true for no reason
+    bool setValue(const int128_t& v) override {
+        i_ = v;
+        return (true);
+    }
+
+    /// @brief Converts the Element to JSON format and appends it to the given
+    /// stringstream.
+    void toJSON(std::ostream& ss) const override;
+
+    /// @brief Checks whether the other Element is equal.
+    ///
+    /// @return true if the other ElementPtr has the same value and the same
+    /// type (or a different and compatible type), false otherwise.
+    bool equals(const Element& other) const override;
+
+private:
+    /// @brief the underlying stored value
+    int128_t i_;
+};
+
 class DoubleElement : public Element {
     double d;
 
