@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -65,7 +65,7 @@ const uint8_t AUTH_KEY_LEN = 16;
 /// @brief Maximum length of authentication keys (coded in hexadecimal).
 const size_t TEXT_AUTH_KEY_LEN = AUTH_KEY_LEN * 2;
 
-/// @brief HostID (used only when storing in MySQL, PostgreSQL or Cassandra)
+/// @brief HostID (used only when storing in MySQL or PostgreSQL backends)
 typedef uint64_t HostID;
 
 /// @brief Authentication keys.
@@ -108,7 +108,7 @@ public:
     ///
     /// Set the key value.
     //  If the size is greater than 16 bytes, we resize to 16 bytes.
-    /// Doesnt throw an exception.
+    /// Doesn't throw an exception.
     /// @param key auth key in binary to be stored
     void setAuthKey(const std::vector<uint8_t>& key);
 
@@ -409,6 +409,13 @@ public:
          const std::string& boot_file_name = "",
          const AuthKey& auth_key = AuthKey(""));
 
+    /// @brief Get maximum identifier length.
+    ///
+    /// This method returns the maximum identifier length.
+    ///
+    /// @param type Identifier type.
+    static size_t getIdentifierMaxLength(const IdentifierType& type);
+
     /// @brief Replaces currently used identifier with a new identifier.
     ///
     /// This method sets a new identifier type and value for a host.
@@ -662,16 +669,22 @@ public:
         return (cfg_option6_);
     }
 
+    /// @brief Encapsulates host-specific options with their suboptions.
+    ///
+    /// This function must be called before the server returns host-specific
+    /// DHCP options to the client.
+    void encapsulateOptions() const;
+
     /// @brief Returns information about the host in the textual format.
     std::string toText() const;
 
-    /// @brief Sets Host ID (primary key in MySQL, PostgreSQL and Cassandra backends)
+    /// @brief Sets Host ID (primary key in MySQL and PostgreSQL backends)
     /// @param id HostId value
     void setHostId(HostID id) {
         host_id_ = id;
     }
 
-    /// @brief Returns Host ID (primary key in MySQL, PostgreSQL and Cassandra backends)
+    /// @brief Returns Host ID (primary key in MySQL and PostgreSQL backends)
     /// @return id HostId value (or 0 if not set)
     HostID getHostId() const {
         return (host_id_);
@@ -716,6 +729,15 @@ public:
         return(key_);
     }
 
+protected:
+
+    /// @brief Set the identifier type.
+    ///
+    /// @note for test only!
+    ///
+    /// @param type Identifier type.
+    void setIdentifierType(const IdentifierType& type);
+
 private:
 
     /// @brief Adds new client class for DHCPv4 or DHCPv6.
@@ -757,7 +779,7 @@ private:
     std::string boot_file_name_;
 
     /// @brief HostID (a unique identifier assigned when the host is stored in
-    ///     MySQL, PostgreSQL or Cassandra)
+    /// MySQL or PostgreSQL backends)
     uint64_t host_id_;
 
     /// @brief Pointer to the DHCPv4 option data configuration for this host.
