@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,18 +11,14 @@
 #include <dhcp/duid.h>
 #include <dhcp/option.h>
 #include <dhcp/hwaddr.h>
+#include <dhcpsrv/subnet_id.h>
 #include <cc/user_context.h>
 #include <cc/cfg_to_element.h>
+#include <util/dhcp_space.h>
 
 namespace isc {
 namespace dhcp {
 
-/// @brief Unique identifier for a subnet (both v4 and v6)
-///
-/// Let's copy SubnetID definition from subnet.h. We can't include it directly,
-/// because subnet.h needs Lease::Type, so it includes lease.h
-typedef uint32_t SubnetID;
-
 struct Lease;
 
 /// @brief Pointer to the lease object.
@@ -97,6 +93,13 @@ struct Lease : public isc::data::UserContext, public isc::data::CfgToElement {
     /// @param fqdn_rev If true, reverse DNS update is performed for a lease.
     /// @param hostname FQDN of the client which gets the lease.
     /// @param hwaddr Hardware/MAC address
+    ///
+    /// @note When creating a new Lease object, current_cltt_ matches cltt_ and
+    /// current_valid_lft_ matches valid_lft_. Any update operation that changes
+    /// cltt_ or valid_lft_ in the database must also update the current_cltt_
+    /// and current_valid_lft_ after the database response so that additional
+    /// operations can be performed on the same object. Failing to do so will
+    /// result in the new actions to be rejected by the database.
     Lease(const isc::asiolink::IOAddress& addr,
           uint32_t valid_lft, SubnetID subnet_id, time_t cltt,
           const bool fqdn_fwd, const bool fqdn_rev,
@@ -106,6 +109,11 @@ struct Lease : public isc::data::UserContext, public isc::data::CfgToElement {
     /// @brief Destructor
     virtual ~Lease() {}
 
+    /// @brief Returns Lease type
+    ///
+    /// One of normal address, temporary address, or prefix, or V4
+    virtual Lease::Type getType() const = 0;
+
     /// @brief IPv4 ot IPv6 address
     ///
     /// IPv4, IPv6 address or, in the case of a prefix delegation, the prefix.
@@ -116,10 +124,17 @@ struct Lease : public isc::data::UserContext, public isc::data::CfgToElement {
     /// Expressed as number of seconds since cltt.
     uint32_t valid_lft_;
 
-    /// @brief Old valid lifetime
+    /// @brief Current valid lifetime
     ///
     /// Expressed as number of seconds since cltt before update.
-    uint32_t old_valid_lft_;
+    uint32_t current_valid_lft_;
+
+    /// @brief Remaining valid lifetime
+    ///
+    /// Expressed as number of seconds since current time, also
+    /// valid lifetime - age where age is old cltt - new cltt.
+    /// The value 0 is used for the "cannot be reused" condition.
+    uint32_t reuseable_valid_lft_;
 
     /// @brief Client last transmission time
     ///
@@ -127,17 +142,22 @@ struct Lease : public isc::data::UserContext, public isc::data::CfgToElement {
     /// client was received.
     time_t cltt_;
 
-    /// @brief Old client last transmission time
+    /// @brief Current client last transmission time
     ///
     /// Specifies a timestamp giving the time when the last transmission from a
     /// client was received before update.
-    time_t old_cltt_;
+    time_t current_cltt_;
 
     /// @brief Subnet identifier
     ///
     /// Specifies the identification of the subnet to which the lease belongs.
     SubnetID subnet_id_;
 
+    /// @brief The pool id
+    ///
+    /// Specifies the identification of the pool from a subnet to which the lease belongs.
+    uint32_t pool_id_;
+
     /// @brief Client hostname
     ///
     /// This field is in lower case and may be empty.
@@ -230,6 +250,27 @@ struct Lease : public isc::data::UserContext, public isc::data::CfgToElement {
     /// Avoid a clang spurious error
     using isc::data::CfgToElement::toElement;
 
+    /// Sync lease current expiration time with new value from another lease,
+    /// so that additional operations can be done without performing extra read
+    /// from the database.
+    ///
+    /// @note The lease current expiration time is represented by the
+    /// @ref current_cltt_ and  @ref current_valid_lft_ and the new value by
+    /// @ref cltt_ and @ref valid_lft_
+    ///
+    /// @param from The lease with latest value of expiration time.
+    /// @param [out] to The lease that needs to be updated.
+    static void syncCurrentExpirationTime(const Lease& from, Lease& to);
+
+    /// Update lease current expiration time with new value,
+    /// so that additional operations can be done without performing extra read
+    /// from the database.
+    ///
+    /// @note The lease current expiration time is represented by the
+    /// @ref current_cltt_ and  @ref current_valid_lft_ and the new value by
+    /// @ref cltt_ and @ref valid_lft_
+    void updateCurrentExpirationTime();
+
 protected:
 
     /// @brief Sets common (for v4 and v6) properties of the lease object.
@@ -291,7 +332,7 @@ struct Lease4 : public Lease {
     /// @brief Constructor.
     ///
     /// @param address IPv4 address.
-    /// @param hw_address Pointer to client's HW addresss.
+    /// @param hw_address Pointer to client's HW address.
     /// @param client_id  pointer to the client id structure.
     /// @param valid_lifetime Valid lifetime value.
     /// @param cltt Timestamp when the lease is acquired, renewed.
@@ -313,14 +354,17 @@ struct Lease4 : public Lease {
     /// @brief Default constructor
     ///
     /// Initialize fields that don't have a default constructor.
-    Lease4() : Lease(0, 0, 0, 0, false, false, "", HWAddrPtr())
-    {
-    }
+    Lease4();
 
-    /// @brief Copy constructor
+    /// @brief Returns Lease type
     ///
-    /// @param other the @c Lease4 object to be copied.
-    Lease4(const Lease4& other);
+    /// Since @c Lease does not define a member for lease type, we implement this
+    /// so we don't store the same value in a billion v4 lease instances.
+    ///
+    /// @return Lease::TYPE_V4
+    virtual Lease::Type getType() const {
+        return (Lease::TYPE_V4);
+    }
 
     /// @brief Returns name of the lease states specific to DHCPv4.
     ///
@@ -406,11 +450,6 @@ struct Lease4 : public Lease {
     bool belongsToClient(const HWAddrPtr& hw_address,
                          const ClientIdPtr& client_id) const;
 
-    /// @brief Assignment operator.
-    ///
-    /// @param other the @c Lease4 object to be assigned.
-    Lease4& operator=(const Lease4& other);
-
     /// @brief Compare two leases for equality
     ///
     /// @param other lease6 object with which to compare
@@ -446,6 +485,12 @@ struct Lease4 : public Lease {
     static Lease4Ptr fromElement(const data::ConstElementPtr& element);
 
     /// @todo: Add DHCPv4 failover related fields here
+
+    /// @brief Remote identifier for Bulk Lease Query
+    std::vector<uint8_t> remote_id_;
+
+    /// @brief Relay identifier for Bulk Lease Query
+    std::vector<uint8_t> relay_id_;
 };
 
 /// @brief A collection of IPv4 leases.
@@ -487,12 +532,30 @@ struct Lease6 : public Lease {
     /// @brief Client identifier
     DuidPtr duid_;
 
-    /// @brief preferred lifetime
+    /// @brief Preferred lifetime
     ///
     /// This parameter specifies the preferred lifetime since the lease was
     /// assigned or renewed (cltt), expressed in seconds.
     uint32_t preferred_lft_;
 
+    /// @brief Remaining preferred lifetime
+    ///
+    /// Expressed as number of seconds since current time, also
+    /// preferred lifetime - age where age is old cltt - new cltt.
+    /// This parameter is used only when reuseable_valid_lft_ is not zero,
+    /// i.e. when the lease can be reused.
+    uint32_t reuseable_preferred_lft_;
+
+    /// @brief Action on extended info tables.
+    typedef enum {
+        ACTION_IGNORE, ///< ignore extended info,
+        ACTION_DELETE, ///< delete reference to the lease
+        ACTION_UPDATE  ///< update extended info tables.
+    } ExtendedInfoAction;
+
+    /// @brief Record the action on extended info tables in the lease.
+    ExtendedInfoAction extended_info_action_;
+
     /// @todo: Add DHCPv6 failover related fields here
 
     /// @brief Constructor
@@ -535,6 +598,16 @@ struct Lease6 : public Lease {
     /// Initialize fields that don't have a default constructor.
     Lease6();
 
+    /// @brief Returns Lease type
+    ///
+    /// Since @c Lease does not define a member for lease type, we implement this
+    /// so code that only has LeasePtr can see what it has.
+    ///
+    /// @return Type of lease
+    virtual Lease::Type getType() const {
+        return (type_);
+    }
+
     /// @brief Returns name of the lease states specific to DHCPv6.
     ///
     /// @todo Currently it simply returns common states for DHCPv4 and DHCPv6.
@@ -596,7 +669,6 @@ typedef boost::shared_ptr<const Lease6> ConstLease6Ptr;
 /// @brief A collection of IPv6 leases.
 typedef std::vector<Lease6Ptr> Lease6Collection;
 
-
 /// @brief A shared pointer to the collection of IPv6 leases.
 typedef boost::shared_ptr<Lease6Collection> Lease6CollectionPtr;
 
@@ -609,7 +681,33 @@ typedef boost::shared_ptr<Lease6Collection> Lease6CollectionPtr;
 std::ostream&
 operator<<(std::ostream& os, const Lease& lease);
 
-}; // end of isc::dhcp namespace
-}; // end of isc namespace
+/// @brief adapters for linking templates to qualified names
+/// @{
+namespace {
+
+template <isc::util::DhcpSpace D>
+struct AdapterLease {};
+
+template <>
+struct AdapterLease<isc::util::DHCPv4> {
+    using type = Lease4;
+};
+
+template <>
+struct AdapterLease<isc::util::DHCPv6> {
+    using type = Lease6;
+};
+
+}  // namespace
+
+template <isc::util::DhcpSpace D>
+using LeaseT = typename AdapterLease<D>::type;
+
+template <isc::util::DhcpSpace D>
+using LeaseTPtr = boost::shared_ptr<LeaseT<D>>;
+/// @}
+
+}  // end of isc::dhcp namespace
+}  // end of isc namespace
 
 #endif // LEASE_H
